from time import time, sleep
import os

def runBatchBatchs(fName, benches, profiles, preRuns):
    start = time()
    info = {}
    for bench, profile, preRun in zip(benches, profiles, preRuns):
        os.system("date")
        benchProg = Program(**bench)
        profileProg = Program(**profile)
        if preRun:
            preRunProgs = [Program(**pr) for pr in preRun]
            test = Batch(benchProg, profileProg, preRunProgs)
        else:
            test = Batch(benchProg, profileProg)
        data = test.run() # contains {name:{}, name:{},} ...
        if data['title'] in info:
            info[data['title']].append(data)
        else:
            info[data['title']] = [data]
        f = open('./files/' + fName + '.txt','w')
        f.write(repr(info))
        f.close()
        print ''
            
    print 'FINISHED:', fName, '\n'
    print 'Time to completion:', str(time() - start), 'seconds'

def runSysstatBatch(fName, argTemplate, iNum=1, isRead=True, xStart=2, xStop=1025, xStep=4):
    xList = range(xStart, xStop, xStep)
    fNameTag = 'oprof-%d_%d_%d' % (xStart, xStop, xStep)
    ssh = {'user':'ben-kvm', 'ip':'160.91.244.130', 'pw':'databas3'} #130
    bench = {'title':'virtual', 
             'name':'iozone', 
             'args':'-i 1 -s 1024m -r 512k -+n -f /media/disk/io.tmp',
             'ssh':{}}
    prof = {'title':'virtual',
            'name':'vmstat',
            'args':'1',
            'ssh':{}}
    preRun = {'title':'iozone write file',
              'name':'iozone',
              'args':'-i 0 -s 1024m -r 512k -+n -f /media/disk/io.tmp',
              'ssh':{}}

    profiles = [prof for i in range(len(xList)*2)]
    
    benches, preRuns = [], []
    for i in xList:
        for sshConfig in [ssh, {}]:
            bench['args'] = argTemplate % (iNum, i)
            preRun['args'] = argTemplate % (0, i) + ' -w'
            bench['ssh'] = sshConfig
            preRun['ssh'] = sshConfig
            if sshConfig:
                bench['title'] = 'virtual'
            else:
                bench['title'] = 'native'
                
            benches.append(bench.copy())
            if isRead:
                preRuns.append(preRun.copy())    
            else:
                #preRuns.append(dict((k,v[1:]) for k,v in preRun.iteritems()))
                preRuns.append({})
    runBatchBatchs(fName + fNameTag, benches, profiles, preRuns)

def runOProfileBatch(fName, argTemplate, iNum=1, isRead=True, xStart=4, xStop=1025, xStep=4):
    xList = range(xStart, xStop, xStep)
    fNameTag = '-2-oprof-%d_%d_%d' % (xStart, xStop, xStep)
    ssh = {'user':'ben-kvm', 'ip':'160.91.244.130', 'pw':'databas3'} #130
    bench = {'title':'virtual', 
             'name':'iozone', 
             'args':'-i 1 -s 1024m -r 512k -+n -f /media/disk/io.tmp',
             'ssh':{}}
    prof = {'title':'profiler',
            'name':'opcontrol',
            'args':'--vmlinux=/home/ben/linux-2.6.25-9/vmlinux --callgraph=3  --start',
            'ssh':{},
            'stopCmd':['opcontrol --shutdown',
                       'opreport  -l --image-path=/home/ben/linux-2.6.25-9/,/usr/local/bin/ --merge tgid -t 0.1']}

    preRun = [{'title':'iozone write file',
              'name':'iozone',
              'args':'-i 0 -s 1024m -r 512k -+n -f /media/disk/io.tmp',
              'ssh':{}}]

    preRun.append({'title':'oprof setup 0',
                   'name':'opcontrol',
                   'args':'--reset',
                   'ssh':{}})
    preRun.append({'title':'oprof setup 1',
                   'name':'opcontrol',
                   'args':'--deinit',
                   'ssh':{}})
    preRun.append({'title':'oprof setup 2',
                   'name':'opcontrol',
                   'args':'--init',
                   'ssh':{}})
    preRun.append({'title':'oprof setup 3',
                   'name':'opcontrol',
                   'args':'--event=DTLB_MISSES:10000',
                   'ssh':{}})

    profiles = [prof for i in range(len(xList)*2)]
    
    benches, preRuns = [], []
    
    for sshConfig in [ssh, {}]:
        for i in xList:
            bench['args'] = argTemplate % (iNum, i)
            preRun[0]['args'] = argTemplate % (0, i) + ' -w'
            bench['ssh'] = sshConfig
            preRun[0]['ssh'] = sshConfig
            if sshConfig:
                bench['title'] = 'virtual'
            else:
                bench['title'] = 'native'
                
            benches.append(bench.copy())
            if isRead:
                preRuns.append(preRun)    
            else:
                preRuns.append(preRun[1:])
                #preRuns.append({})
    runBatchBatchs(fName + fNameTag, benches, profiles, preRuns)

def runVmstatIozoneBatchs():    
    '''
    #### FWRITE file size
    argTemplate = '-i %d -s %dm -r 512k -+n -f /media/disk/io.tmp'
    runSysstatBatch('kvmFWrite', argTemplate, iNum=6, isRead=False)
    
    #### SEQUENTIAL READ file size
    argTemplate = '-i %d -s %dm -r 512k -+n -f /media/disk/io.tmp'
    runSysstatBatch('kvmRead', argTemplate, iNum=1, isRead=True)
    
    #### FREAD file size
    argTemplate = '-i %d -s %dm -r 512k -+n -f /media/disk/io.tmp'
    runSysstatBatch('kvmFRead', argTemplate, iNum=7, isRead=True)

    #### RANDOM file size
    argTemplate = '-i %d -s %dm -r 512k -+n -f /media/disk/io.tmp'
    runSysstatBatch('kvmRandom', argTemplate, iNum=3, isRead=True)
    
    #### SEQUENTIAL WRITE file size 
    argTemplate = '-i %d -s %dm -r 512k -+n -f /media/disk/io.tmp'
    runSysstatBatch('kvmWrite', argTemplate, iNum=0, isRead=False)
    
    #-----------------------------------
    #### FWRITE file size
    argTemplate = '-i %d -s %dm -r 512k -+n -f /media/disk/io.tmp -I -o'
    runSysstatBatch('kvmFWriteNoBuf', argTemplate, iNum=6, isRead=False)
    
    #### SEQUENTIAL READ file size
    argTemplate = '-i %d -s %dm -r 512k -+n -f /media/disk/io.tmp -I -o'
    runSysstatBatch('kvmReadNoBuf', argTemplate, iNum=1, isRead=True)
    
    #### FREAD file size
    argTemplate = '-i %d -s %dm -r 512k -+n -f /media/disk/io.tmp -I -o'
    runSysstatBatch('kvmFReadNoBuf', argTemplate, iNum=7, isRead=True)

    #### RANDOM file size
    argTemplate = '-i %d -s %dm -r 512k -+n -f /media/disk/io.tmp -I -o'
    runSysstatBatch('kvmRandomNoBuf', argTemplate, iNum=3, isRead=True)
    
    #### SEQUENTIAL WRITE file size 
    argTemplate = '-i %d -s %dm -r 512k -+n -f /media/disk/io.tmp -I -o'
    runSysstatBatch('kvmWriteNoBuf', argTemplate, iNum=0, isRead=False)
    #-----------------------------------
    '''
    
    #### FWRITE file size
    argTemplate = '-i %d -s %dm -r 512k -+n -f /media/disk/io.tmp'
    runOProfileBatch('kvmFWrite', argTemplate, iNum=6, isRead=False, xStep=4)
    
    #### SEQUENTIAL READ file size
    argTemplate = '-i %d -s %dm -r 512k -+n -f /media/disk/io.tmp'
    runOProfileBatch('kvmRead', argTemplate, iNum=1, isRead=True, xStep=4)
    
    #### FREAD file size
    argTemplate = '-i %d -s %dm -r 512k -+n -f /media/disk/io.tmp'
    runOProfileBatch('kvmFRead', argTemplate, iNum=7, isRead=True, xStep=4)

    #### RANDOM file size
    argTemplate = '-i %d -s %dm -r 512k -+n -f /media/disk/io.tmp'
    runOProfileBatch('kvmRandom', argTemplate, iNum=3, isRead=True, xStep=4)
    
    #### SEQUENTIAL WRITE file size 
    argTemplate = '-i %d -s %dm -r 512k -+n -f /media/disk/io.tmp'
    runOProfileBatch('kvmWrite', argTemplate, iNum=0, isRead=False, xStep=4)
    
    #-----------------------------------
    #### FWRITE file size
    argTemplate = '-i %d -s %dm -r 512k -+n -f /media/disk/io.tmp -I -o'
    runOProfileBatch('kvmFWriteNoBuf', argTemplate, iNum=6, isRead=False, xStep=4)
    
    #### SEQUENTIAL READ file size
    argTemplate = '-i %d -s %dm -r 512k -+n -f /media/disk/io.tmp -I -o'
    runOProfileBatch('kvmReadNoBuf', argTemplate, iNum=1, isRead=True, xStep=4)
    
    #### FREAD file size
    argTemplate = '-i %d -s %dm -r 512k -+n -f /media/disk/io.tmp -I -o'
    runOProfileBatch('kvmFReadNoBuf', argTemplate, iNum=7, isRead=True, xStep=4)

    #### RANDOM file size
    argTemplate = '-i %d -s %dm -r 512k -+n -f /media/disk/io.tmp -I -o'
    runOProfileBatch('kvmRandomNoBuf', argTemplate, iNum=3, isRead=True, xStep=4)
    
    #### SEQUENTIAL WRITE file size 
    argTemplate = '-i %d -s %dm -r 512k -+n -f /media/disk/io.tmp -I -o'
    runOProfileBatch('kvmWriteNoBuf', argTemplate, iNum=0, isRead=False, xStep=4)
    #-----------------------------------

    #### SEQUENTIAL READ record size
    argTemplate = '-i %d -s 1024m -r %dk -+n -f /media/disk/io.tmp'
    runOProfileBatch('kvmReadRecordSize', argTemplate, iNum=1, isRead=True, xStep=1)
    
    #### SEQUENTIAL WRITE record size
    argTemplate = '-i %d -s 512m -r %dk -+n -f /media/disk/io.tmp'
    runOProfileBatch('kvmWriteRecordSize', argTemplate, iNum=0, isRead=False, xStep=1)

    #### FWRITE record size
    argTemplate = '-i %d -s 1024m -r %dk -+n -f /media/disk/io.tmp'
    runOProfileBatch('kvmFWriteRecordSize', argTemplate, iNum=6, isRead=False, xStep=1)

    #### FREAD record size
    argTemplate = '-i %d -s 1024m -r %dk -+n -f /media/disk/io.tmp'
    runOProfileBatch('kvmFReadRecordSize', argTemplate, iNum=7, isRead=True, xStep=1)

    #### RANDOM record size
    argTemplate = '-i %d -s 1024m -r %dk -+n -f /media/disk/io.tmp'
    runOProfileBatch('kvmRandomRecordSize', argTemplate, iNum=3, isRead=True, xStep=1)


